@@ -4,6 +4,7 @@
 #include "libfilezilla.hpp"
 
 #include <string>
+#include <vector>
 
 /** \file
  * \brief String types and assorted functions.
@@ -47,7 +48,7 @@ native_string FZ_PUBLIC_SYMBOL to_native(std::wstring const& in);
 
 /** \brief Locale-sensitive stricmp
  *
- * Like std::string::strcmp but case-insensitive, respectinge locale.
+ * Like std::string::strcmp but case-insensitive, respecting locale.
  *
  * \note does not handle embedded null
  */
@@ -106,6 +107,14 @@ std::wstring FZ_PUBLIC_SYMBOL to_wstring(std::string const& in);
 /// Returns identity, that way to_wstring can be called with native_string.
 inline std::wstring FZ_PUBLIC_SYMBOL to_wstring(std::wstring const& in) { return in; }
 
+/// Converts from arithmetic type to std::wstring
+template<typename Arg>
+inline typename std::enable_if<std::is_arithmetic<std::decay_t<Arg>>::value, std::wstring>::type to_wstring(Arg && arg)
+{
+	return std::to_wstring(std::forward<Arg>(arg));
+}
+
+
 /** \brief Converts from std::string in UTF-8 into std::wstring
  *
  * \return the converted string on success. On failure an empty string is returned.
@@ -123,6 +132,14 @@ std::string FZ_PUBLIC_SYMBOL to_string(std::wstring const& in);
 /// Returns identity, that way to_string can be called with native_string.
 inline std::string FZ_PUBLIC_SYMBOL to_string(std::string const& in) { return in; }
 
+/// Converts from arithmetic type to std::string
+template<typename Arg>
+inline typename std::enable_if<std::is_arithmetic<std::decay_t<Arg>>::value, std::string>::type to_string(Arg && arg)
+{
+	return std::to_string(std::forward<Arg>(arg));
+}
+
+
 /// Returns length of 0-terminated character sequence. Works with both narrow and wide-characters.
 template<typename Char>
 size_t strlen(Char const* str) {
@@ -164,14 +181,35 @@ int hex_char_to_int(Char c)
 		return c - '0';
 }
 
-/** \brief Convert integer to string. */
-template<typename String, typename Int>
-inline typename std::enable_if<std::is_same<String, std::string>::value, std::string>::type
-convert(Int i) { return std::to_string(i); };
+/** \brief Converts an integer to the corresponding lowercase hex digit
+*
+* Example: 9 becomes '9', 11 becomes 'b'
+*
+* Undefined output if input is less than 0 or larger than 15
+*/
+template<typename Char = char, bool Lowercase = true>
+Char int_to_hex_char(int d)
+{
+	if (d > 9) {
+		return (Lowercase ? 'a' : 'A') + d - 10;
+	}
+	else {
+		return '0' + d;
+	}
+}
 
-template<typename String, typename Int>
-inline typename std::enable_if<std::is_same<String, std::wstring>::value, std::wstring>::type
-convert(Int i) { return std::to_wstring(i); };
+/// Calls either fz::to_string or fz::to_wstring depending on the passed template argument
+template<typename String, typename Arg>
+inline auto toString(Arg&& arg) -> typename std::enable_if<std::is_same<String, std::string>::value, decltype(to_string(std::forward<Arg>(arg)))>::type
+{
+	return to_string(std::forward<Arg>(arg));
+}
+
+template<typename String, typename Arg>
+inline auto toString(Arg&& arg) -> typename std::enable_if<std::is_same<String, std::wstring>::value, decltype(to_wstring(std::forward<Arg>(arg)))>::type
+{
+	return to_wstring(std::forward<Arg>(arg));
+}
 
 #if !defined(fzT) || defined(DOXYGEN)
 #ifdef FZ_WINDOWS
@@ -189,7 +227,7 @@ convert(Int i) { return std::to_wstring(i); };
 #endif
 #endif
 
-/// Returns the function argument of the type matching the template argument. \sa fzS
+ /// Returns the function argument of the type matching the template argument. \sa fzS
 template<typename Char>
 Char const* choose_string(char const* c, wchar_t const* w);
 
@@ -208,8 +246,78 @@ template<> inline wchar_t const* choose_string(char const*, wchar_t const* w) {
  *   }
  * \endcode
  */
-#define fzS(Char, s) choose_string<Char>(s, L ## s)
+#define fzS(Char, s) fz::choose_string<Char>(s, L ## s)
 #endif
+
+ /// Returns \c in with all occurrences of \c find in the input string replaced with \c replacement
+std::string FZ_PUBLIC_SYMBOL replaced_substrings(std::string const& in, std::string const& find, std::string const& replacement);
+std::wstring FZ_PUBLIC_SYMBOL replaced_substrings(std::wstring const& in, std::wstring const& find, std::wstring const& replacement);
+
+/// Modifies \c in, replacing all occurrences of \c find with \c replacement
+void FZ_PUBLIC_SYMBOL replace_substrings(std::string& in, std::string const& find, std::string const& replacement);
+void FZ_PUBLIC_SYMBOL replace_substrings(std::wstring& in, std::wstring const& find, std::wstring const& replacement);
+
+/// Tokenizes string. Returns all non-empty substrings
+template<typename String, typename Delim, typename Container = std::vector<String>>
+Container strtok(String const& s, Delim const& delims)
+{
+	Container ret;
+
+	typename String::size_type start{}, pos{};
+	do {
+		pos = s.find_first_of(delims, start);
+
+		// Not found, we're at ends;
+		if (pos == String::npos) {
+			if (start + 1 < s.size()) {
+				ret.emplace_back(s.substr(start));
+			}
+		}
+		else if (pos > start + 1) {
+			// Non-empty substring
+			ret.emplace_back(s.substr(start, pos - start));
+		}
+		start = pos + 1;
+	} while (pos != String::npos);
+
+	return ret;
+}
+
+
+/// \brief Encodes raw input string to base64
+std::string FZ_PUBLIC_SYMBOL base64_encode(std::string const& in);
+
+/// \brief Decodes base64, ignores whitespace. Returns empty string on invalid input.
+std::string FZ_PUBLIC_SYMBOL base64_decode(std::string const& in);
+
+// Converts string to integral type T. If string is not convertible, T() is returned.
+template<typename T, typename String>
+T to_integral(String const& s)
+{
+	T ret{};
+
+	auto it = s.cbegin();
+	if (it != s.cend() && (*it == '-' || *it == '+')) {
+		++it;
+	}
+
+	for (; it != s.cend(); ++it) {
+		auto const& c = *it;
+		if (c < '0' || c > '9') {
+			return T();
+		}
+		ret *= 10;
+		ret += c - '0';
+	}
+
+	if (!s.empty() && s.front() == '-') {
+		return ret *= static_cast<T>(-1);
+	}
+	else {
+		return ret;
+	}
+}
+
 }
 
 #endif
